@@ -16,6 +16,19 @@
  *   License along with this program; if not, write to the                 *
  *   Free Software Foundation, Inc.,                                       *
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
+ *                                                                         *
+ *   In addition, as a special exception, the copyright holders give       *
+ *   permission to link the code of portions of this program with the      *
+ *   OpenSSL library under certain conditions as described in each         *
+ *   individual source file, and distribute linked combinations            *
+ *   including the two.                                                    *
+ *   You must obey the GNU General Public License in all respects          *
+ *   for all of the code used other than OpenSSL.  If you modify           *
+ *   file(s) with this exception, you may extend this exception to your    *
+ *   version of the file(s), but you are not obligated to do so.  If you   *
+ *   do not wish to do so, delete this exception statement from your       *
+ *   version.  If you delete this exception statement from all source      *
+ *   files in the program, then also delete it here.                       *
  ***************************************************************************/
 
 #include "PdfColor.h"
@@ -539,6 +552,7 @@ PdfColorSeparation::PdfColorSeparation( const std::string & sName, double dDensi
             break;
         }
         case ePdfColorSpace_Unknown:
+        case ePdfColorSpace_Indexed:
         default:
         {
             PODOFO_RAISE_ERROR( ePdfError_InvalidEnumValue );
@@ -610,6 +624,7 @@ PdfColor PdfColor::ConvertToGrayScale() const
             break;
         }
         case ePdfColorSpace_CieLab:
+        case ePdfColorSpace_Indexed:
         case ePdfColorSpace_Unknown:
         {
             PODOFO_RAISE_ERROR( ePdfError_CannotConvertColor );
@@ -673,6 +688,7 @@ PdfColor PdfColor::ConvertToRGB() const
             break;
         }
         case ePdfColorSpace_CieLab:
+        case ePdfColorSpace_Indexed:
         case ePdfColorSpace_Unknown:
         {
             PODOFO_RAISE_ERROR( ePdfError_CannotConvertColor );
@@ -723,6 +739,7 @@ PdfColor PdfColor::ConvertToCMYK() const
         }
         case ePdfColorSpace_Separation:
         case ePdfColorSpace_CieLab:
+        case ePdfColorSpace_Indexed:
         case ePdfColorSpace_Unknown:
         {
             PODOFO_RAISE_ERROR( ePdfError_CannotConvertColor );
@@ -774,6 +791,7 @@ PdfArray PdfColor::ToArray() const
             array.push_back( m_separationDensity );
             break;
         }
+        case ePdfColorSpace_Indexed:
         case ePdfColorSpace_Unknown:
         {
             PODOFO_RAISE_ERROR( ePdfError_CannotConvertColor );
@@ -975,9 +993,6 @@ PdfObject* PdfColor::BuildColorSpace( PdfVecObjects* pOwner ) const
                     data[0] = 0;
                     data[1] = static_cast<char> (m_uColor.gray);
 
-                    PdfMemoryInputStream stream( data, 1*2 );
-                    csTintFunc->GetStream()->Set( &stream );
-
                     PdfArray range;
                     range.push_back( static_cast<pdf_int64>(0) );
                     range.push_back( static_cast<pdf_int64>(1) );
@@ -987,6 +1002,9 @@ PdfObject* PdfColor::BuildColorSpace( PdfVecObjects* pOwner ) const
                     size.push_back( static_cast<pdf_int64>(2) );
                     csTintFunc->GetDictionary().AddKey( "Size", size );
 
+                    PdfMemoryInputStream stream( data, 1*2 );
+                    csTintFunc->GetStream()->Set( &stream );
+
                     PdfArray csArr;
                     csArr.push_back( PdfName("Separation") );
                     csArr.push_back( PdfName( m_separationName ) );
@@ -1009,9 +1027,6 @@ PdfObject* PdfColor::BuildColorSpace( PdfVecObjects* pOwner ) const
                     data[4] = static_cast<char> (m_uColor.rgb[1] * 255);
                     data[5] = static_cast<char> (m_uColor.rgb[2] * 255);
 
-                    PdfMemoryInputStream stream( data, 3*2 );
-                    csTintFunc->GetStream()->Set( &stream );
-
                     PdfArray range;
                     range.push_back( static_cast<pdf_int64>(0) );
                     range.push_back( static_cast<pdf_int64>(1) );
@@ -1025,6 +1040,9 @@ PdfObject* PdfColor::BuildColorSpace( PdfVecObjects* pOwner ) const
                     size.push_back( static_cast<pdf_int64>(2) );
                     csTintFunc->GetDictionary().AddKey( "Size", size );
 
+                    PdfMemoryInputStream stream( data, 3*2 );
+                    csTintFunc->GetStream()->Set( &stream );
+
                     PdfArray csArr;
                     csArr.push_back( PdfName("Separation") );
                     csArr.push_back( PdfName( m_separationName ) );
@@ -1049,9 +1067,6 @@ PdfObject* PdfColor::BuildColorSpace( PdfVecObjects* pOwner ) const
                     data[6] = static_cast<char> (m_uColor.cmyk[2] * 255);
                     data[7] = static_cast<char> (m_uColor.cmyk[3] * 255);
 
-                    PdfMemoryInputStream stream( data, 4*2 );
-                    csTintFunc->GetStream()->Set( &stream );
-
                     PdfArray range;
                     range.push_back( static_cast<pdf_int64>(0) );
                     range.push_back( static_cast<pdf_int64>(1) );
@@ -1073,6 +1088,9 @@ PdfObject* PdfColor::BuildColorSpace( PdfVecObjects* pOwner ) const
                     csArr.push_back( PdfName("DeviceCMYK") );
                     csArr.push_back( csTintFunc->Reference() );
 
+                    PdfMemoryInputStream stream( data, 4*2 );
+                    csTintFunc->GetStream()->Set( &stream ); // set stream as last, so that it will work with PdfStreamedDocument
+
                     PdfObject* csp = pOwner->CreateObject( csArr );
 
                     return csp;
@@ -1089,9 +1107,6 @@ PdfObject* PdfColor::BuildColorSpace( PdfVecObjects* pOwner ) const
                     data[4] = static_cast<char> (m_uColor.lab[1] * 255);
                     data[5] = static_cast<char> (m_uColor.lab[2] * 255);
 
-                    PdfMemoryInputStream stream( data, 3*2 );
-                    csTintFunc->GetStream()->Set( &stream );
-
                     PdfArray range;
                     range.push_back( static_cast<pdf_int64>(0) );
                     range.push_back( static_cast<pdf_int64>(1) );
@@ -1105,6 +1120,9 @@ PdfObject* PdfColor::BuildColorSpace( PdfVecObjects* pOwner ) const
                     size.push_back( static_cast<pdf_int64>(2) );
                     csTintFunc->GetDictionary().AddKey( "Size", size );
 
+                    PdfMemoryInputStream stream( data, 3*2 );
+                    csTintFunc->GetStream()->Set( &stream );
+
                     PdfArray csArr;
                     csArr.push_back( PdfName("Separation") );
                     csArr.push_back( PdfName( m_separationName ) );
@@ -1118,6 +1136,7 @@ PdfObject* PdfColor::BuildColorSpace( PdfVecObjects* pOwner ) const
                 break;
 
                 case ePdfColorSpace_Separation:
+                case ePdfColorSpace_Indexed:
                 {
                     break;
                 }
@@ -1165,6 +1184,7 @@ PdfObject* PdfColor::BuildColorSpace( PdfVecObjects* pOwner ) const
         case ePdfColorSpace_DeviceGray:
         case ePdfColorSpace_DeviceRGB:
         case ePdfColorSpace_DeviceCMYK:
+        case ePdfColorSpace_Indexed:
         {
             break;
         }
@@ -1196,6 +1216,10 @@ EPdfColorSpace PdfColor::GetColorSpaceForName( const PdfName & rName )
     {
         ePdfColorSpace = ePdfColorSpace_DeviceCMYK;
     }
+    else if( PdfName("Indexed") == rName ) 
+    {
+        ePdfColorSpace = ePdfColorSpace_Indexed;
+    }
     else
     {
         // TODO: other are not supported at the moment
@@ -1219,6 +1243,8 @@ PdfName PdfColor::GetNameForColorSpace( EPdfColorSpace eColorSpace )
             return PdfName("Separation");
         case ePdfColorSpace_CieLab:
             return PdfName("Lab");
+        case ePdfColorSpace_Indexed:
+            return PdfName("Indexed");
         case ePdfColorSpace_Unknown:
         default:
             PdfError::LogMessage( eLogSeverity_Information, "Unsupported colorspace enum: %i", eColorSpace );
