@@ -67,6 +67,26 @@ ePdfEncryptAlgorithm_RC4V2 |
 ePdfEncryptAlgorithm_AESV2;
 #endif // PODOFO_HAVE_LIBIDN
 
+PdfEncrypt::~PdfEncrypt()
+{
+}
+    
+int PdfEncrypt::GetEnabledEncryptionAlgorithms()
+{
+    return PdfEncrypt::s_nEnabledEncryptionAlgorithms;
+}
+
+void PdfEncrypt::SetEnabledEncryptionAlgorithms(int nEncryptionAlgorithms)
+{
+    PdfEncrypt::s_nEnabledEncryptionAlgorithms = nEncryptionAlgorithms;
+}
+
+bool PdfEncrypt::IsEncryptionEnabled(EPdfEncryptAlgorithm eAlgorithm)
+{
+    return (PdfEncrypt::s_nEnabledEncryptionAlgorithms & eAlgorithm) != 0;
+}
+
+  
 #ifdef PODOFO_HAVE_OPENSSL
 // Default value for P (permissions) = no permission
 #define PERMS_DEFAULT 0xFFFFF0C0
@@ -219,7 +239,7 @@ public:
         if( !lLen )
             return lLen;
         
-        char* pOutputBuffer = static_cast<char*>(malloc( sizeof(char) * lLen ));
+        char* pOutputBuffer = static_cast<char*>(podofo_calloc( lLen, sizeof(char) ));
         if( !pOutputBuffer )
         {
             PODOFO_RAISE_ERROR( ePdfError_OutOfMemory );
@@ -230,7 +250,7 @@ public:
         m_stream.Encrypt( pOutputBuffer, lLen );
         m_pOutputStream->Write( pOutputBuffer, lLen );
         
-        free( pOutputBuffer );
+        podofo_free( pOutputBuffer );
         return lLen;
     }
     
@@ -582,25 +602,6 @@ PdfEncrypt::CreatePdfEncrypt(const PdfEncrypt & rhs )
     return pdfEncrypt;
 }
 
-PdfEncrypt::~PdfEncrypt()
-{
-}
-    
-int PdfEncrypt::GetEnabledEncryptionAlgorithms()
-{
-    return PdfEncrypt::s_nEnabledEncryptionAlgorithms;
-}
-
-void PdfEncrypt::SetEnabledEncryptionAlgorithms(int nEncryptionAlgorithms)
-{
-    PdfEncrypt::s_nEnabledEncryptionAlgorithms = nEncryptionAlgorithms;
-}
-
-bool PdfEncrypt::IsEncryptionEnabled(EPdfEncryptAlgorithm eAlgorithm)
-{
-    return (PdfEncrypt::s_nEnabledEncryptionAlgorithms & eAlgorithm) != 0;
-}
-
 PdfEncrypt::PdfEncrypt( const PdfEncrypt & rhs )
 {
     m_eAlgorithm = rhs.m_eAlgorithm;
@@ -1547,7 +1548,7 @@ void PdfEncryptSHABase::PreprocessPassword( const std::string &password, unsigne
     len = l > 127 ? 127 : l;
     
     memcpy(outBuf, password_sasl, len);
-    free(password_sasl);
+    free(password_sasl); // Do not change to podofo_free, as memory is allocated by stringprep_profile
 }
 
 void
@@ -1989,7 +1990,7 @@ static void MD5Update(MD5_CTX *ctx, unsigned char const *buf, unsigned len)
  */
 static void MD5Transform(unsigned int buf[4], unsigned int const in[16])
 {
-  register unsigned int a, b, c, d;
+  unsigned int a, b, c, d;
 
   a = buf[0];
   b = buf[1];
@@ -2071,33 +2072,29 @@ static void MD5Transform(unsigned int buf[4], unsigned int const in[16])
 }
 
 PdfEncrypt *
-PdfEncrypt::CreatePdfEncrypt( const std::string & userPassword, 
-                              const std::string & ownerPassword, 
-                              int protection,
-                              EPdfEncryptAlgorithm eAlgorithm, 
-                              EPdfKeyLength eKeyLength )
+PdfEncrypt::CreatePdfEncrypt( const std::string &, 
+                              const std::string &, 
+                              int,
+                              EPdfEncryptAlgorithm, 
+                              EPdfKeyLength )
 {
     PODOFO_RAISE_ERROR_INFO( ePdfError_NotCompiled, "PdfEncrypt::CreatePdfEncrypt: Encryption support was disabled during compile time" );
     return NULL;
 }
 
-PdfEncrypt* PdfEncrypt::CreatePdfEncrypt( const PdfObject* pObject )
+PdfEncrypt* PdfEncrypt::CreatePdfEncrypt( const PdfObject* )
 {
     PODOFO_RAISE_ERROR_INFO( ePdfError_NotCompiled, "PdfEncrypt::CreatePdfEncrypt: Encryption support was disabled during compile time" );
     return NULL;
 }
 
 PdfEncrypt *
-PdfEncrypt::CreatePdfEncrypt(const PdfEncrypt & rhs )  
+PdfEncrypt::CreatePdfEncrypt(const PdfEncrypt & )  
 {
     PODOFO_RAISE_ERROR_INFO( ePdfError_NotCompiled, "PdfEncrypt::CreatePdfEncrypt: Encryption support was disabled during compile time" );
     return NULL;
 }
 
-PdfEncrypt::~PdfEncrypt()
-{
-}
-
 void PdfEncryptMD5Base::GetMD5Binary(const unsigned char* data, int length, unsigned char* digest)
 {
     MD5_CTX ctx;
