@@ -61,6 +61,13 @@ using std::flush;
 #define PDF_XREF_ENTRY_SIZE 20
 #define PDF_XREF_BUF        512
 
+#if defined( PTRDIFF_MAX )
+#define PDF_LONG_MAX PTRDIFF_MAX
+#else
+// only old compilers don't define PTRDIFF_MAX (all 32-bit only?)
+#define PDF_LONG_MAX INT_MAX
+#endif
+
 namespace PoDoFo {
 
 long PdfParser::s_nMaxObjects = std::numeric_limits<long>::max();
@@ -79,7 +86,7 @@ PdfParser::PdfParser( PdfVecObjects* pVecObjects, const char* pszFilename, bool
 }
 
 #ifdef _WIN32
-#if defined(_MSC_VER)  &&  _MSC_VER <= 1200			// nicht fr Visualstudio 6
+#if defined(_MSC_VER)  &&  _MSC_VER <= 1200    // not for MS Visual Studio 6
 #else
 PdfParser::PdfParser( PdfVecObjects* pVecObjects, const wchar_t* pszFilename, bool bLoadOnDemand )
     : PdfTokenizer(), m_vecObjects( pVecObjects )
@@ -158,7 +165,7 @@ void PdfParser::ParseFile( const char* pszFilename, bool bLoadOnDemand )
 }
 
 #ifdef _WIN32
-#if defined(_MSC_VER)  &&  _MSC_VER <= 1200			// nicht fr Visualstudio 6
+#if defined(_MSC_VER)  &&  _MSC_VER <= 1200			// not for MS Visual Studio 6
 #else
 void PdfParser::ParseFile( const wchar_t* pszFilename, bool bLoadOnDemand )
 {
@@ -251,9 +258,7 @@ void PdfParser::Clear()
 void PdfParser::ReadDocumentStructure()
 {
 // Ulrich Arnold 8.9.2009, deactivated because of problems during reading xref's
-   //begin L.K - na test povoleno
-    HasLinearizationDict();
-    //begin L.K
+    // HasLinearizationDict();
     
     // position at the end of the file to search the xref table.
     m_device.Device()->Seek( 0, std::ios_base::end );
@@ -787,22 +792,41 @@ void PdfParser::ReadXRefSubsection( pdf_int64 & nFirstObject, pdf_int64 & nNumOb
 		const int objID = nFirstObject+count;
 #endif // _WIN32
 
-        if( !m_offsets[objID].bParsed )
+        if( static_cast<size_t>(objID) < m_offsets.size() && !m_offsets[objID].bParsed )
         {
+            // don't scan directly into m_offsets since TXRefEntry structure member sizes change between platforms and compilers
+            //
+            //  pdf_long lOffset; // pdf_long is ptrdiff_t: 64 bits on Mac64, Linux64, Win64; 32 bits on Mac32, Linux32, Win32
+            //  long lGeneration; // 64 bits on Mac64, Linux64; 32 bits on Win64, Mac32, Linux32, Win32
+            //  char cUsed;       // always 8 bits
+            //
+            pdf_int64 llOffset = 0;
+            pdf_int64 llGeneration = 0;
+            char cUsed = 0;
+            
+            // XRefEntry is defined in PDF spec section 7.5.4 Cross-Reference Table as
+            // nnnnnnnnnn ggggg n eol
+            // nnnnnnnnnn is 10-digit offset number with max value 9999999999 (bigger than 2**32 = 4GB)
+            // ggggg is a 5-digit generation number with max value 99999 (smaller than 2**17)
+            int read = sscanf( m_buffer.GetBuffer(), "%10" PDF_FORMAT_INT64 " %5" PDF_FORMAT_INT64 " %c%c%c",
+                              &llOffset, &llGeneration, &cUsed, &empty1, &empty2 );
+            
+            if ( read != 5 )
+            {
+                // part of XrefEntry is missing, or i/o error
+                PODOFO_RAISE_ERROR( ePdfError_InvalidXRef );
+            }
+            
+            if ( llOffset > PDF_LONG_MAX )
+            {
+                // pdf_long max size is PTRDIFF_MAX, so throw error if llOffset too big
+                PODOFO_RAISE_ERROR( ePdfError_ValueOutOfRange ); 
+            }
+            
+            m_offsets[objID].lOffset = static_cast<pdf_long>(llOffset);
+            m_offsets[objID].lGeneration = static_cast<long>(llGeneration);
+            m_offsets[objID].cUsed = cUsed;
             m_offsets[objID].bParsed = true;
-#if defined(_WIN64) && defined(_MSC_VER)
-            sscanf( m_buffer.GetBuffer(), "%10" PDF_FORMAT_INT64 " %5ld %c%c%c", 
-                    &(m_offsets[objID].lOffset), 
-                    &(m_offsets[objID].lGeneration), &(m_offsets[objID].cUsed), &empty1, &empty2 );
-#else
-            pdf_int64 tmp1;
-            pdf_int64 tmp2;
-            sscanf( m_buffer.GetBuffer(), "%10" PDF_FORMAT_INT64 " %5ld %c%c%c", 
-                    &tmp1, &tmp2, &(m_offsets[objID].cUsed), &empty1, &empty2 );
-
-            m_offsets[objID].lOffset = static_cast<pdf_long>(tmp1);
-            m_offsets[objID].lGeneration = static_cast<long>(tmp2);
-#endif
        }
 
         ++count;
@@ -833,12 +857,16 @@ void PdfParser::ReadXRefStreamContents( pdf_long lOffset, bool bReadOnlyTrailer
 
     xrefObject.ReadXRefTable();
 
-    // Check for a previous XRef stream
+    // Check for a previous XRefStm or xref table
     if(xrefObject.HasPrevious()) 
     {
         try {
             m_nIncrementalUpdates++;
-            this->ReadXRefStreamContents( xrefObject.GetPreviousOffset(), bReadOnlyTrailer );
+
+            // PDFs that have been through multiple PDF tools may have a mix of xref tables (ISO 32000-1 7.5.4) 
+            // and XRefStm streams (ISO 32000-1 7.5.8.1) and in the Prev chain, 
+            // so call ReadXRefContents (which deals with both) instead of ReadXRefStreamContents 
+            ReadXRefContents( xrefObject.GetPreviousOffset(), bReadOnlyTrailer );
         } catch(PdfError &e) {
             /* Be forgiving, the error happens when an entry in XRef stream points
                to a wrong place (offset) in the PDF file. */
