@@ -109,7 +109,7 @@ PdfString::PdfString( const char* pszString, const PdfEncoding * const pEncoding
         Init( pszString, strlen( pszString ) );
 }
 
-#if defined(_MSC_VER)  &&  _MSC_VER <= 1200			// nicht fr Visualstudio 6
+#if defined(_MSC_VER)  &&  _MSC_VER <= 1200    // not for MS Visual Studio 6
 #else
 PdfString::PdfString( const wchar_t* pszString, pdf_long lLen )
 {
@@ -148,17 +148,22 @@ void PdfString::setFromWchar_t(const wchar_t* pszString, pdf_long lLen )
         {
             // Try to convert to UTF8
             pdf_long   lDest = 5 * lLen; // At max 5 bytes per UTF8 char
-            char*  pDest = static_cast<char*>(malloc( lDest ));
+            char*  pDest = static_cast<char*>(podofo_malloc( lDest ));
+			if (!pDest)
+			{
+				PODOFO_RAISE_ERROR(ePdfError_OutOfMemory);
+			}
+
             size_t cnt   = wcstombs(pDest, pszString, lDest);
             if( cnt != static_cast<size_t>(-1) )
             {
                 // No error
                 InitFromUtf8( reinterpret_cast<pdf_utf8*>(pDest), cnt );
-                free( pDest );
+                podofo_free( pDest );
 	    }
             else
             {
-                free( pDest );
+                podofo_free( pDest );
                 PdfError e( ePdfError_InternalLogic, __FILE__, __LINE__ );
                 e.SetErrorInformation( pszString );
                 throw e;
@@ -257,55 +262,58 @@ void PdfString::SetHexData( const char* pszHex, pdf_long lLen, PdfEncrypt* pEncr
     m_buffer = PdfRefCountedBuffer( lLen % 2 ? ((lLen + 1) >> 1) + 2 : (lLen >> 1) + 2 );
     m_bHex   = true;
     char* pBuffer = m_buffer.GetBuffer();
-    char val;
-    char cDecodedByte = 0;
-    bool bLow = true;
-
-    while( lLen-- ) 
+    if ( pBuffer != NULL )
     {
-        if( PdfTokenizer::IsWhitespace( *pszHex ) )
+        char val;
+        char cDecodedByte = 0;
+        bool bLow = true;
+
+        while( lLen-- ) 
         {
+            if( PdfTokenizer::IsWhitespace( *pszHex ) )
+            {
+                ++pszHex;
+                continue;
+            }
+
+            val = PdfTokenizer::GetHexValue( *pszHex );
+            if( bLow ) 
+            {
+                cDecodedByte = (val & 0x0F);
+                bLow         = false;
+            }
+            else
+            {
+                cDecodedByte = ((cDecodedByte << 4) | val);
+                bLow         = true;
+
+                *pBuffer++ = cDecodedByte;
+            }
+
             ++pszHex;
-            continue;
         }
 
-        val = PdfTokenizer::GetHexValue( *pszHex );
-        if( bLow ) 
+        if( !bLow ) 
         {
-            cDecodedByte = (val & 0x0F);
-            bLow         = false;
-        }
-        else
-        {
-            cDecodedByte = ((cDecodedByte << 4) | val);
-            bLow         = true;
-
+            // an odd number of bytes was read,
+            // so the last byte is 0
             *pBuffer++ = cDecodedByte;
         }
 
-        ++pszHex;
+        *pBuffer++ = '\0';
+        *pBuffer++ = '\0';
+
+        // If the allocated internal buffer is too big (e.g. because of whitespaces in the data)
+        // copy to a smaller buffer so that PdfString::GetLength() will be correct
+        lLen = pBuffer - m_buffer.GetBuffer();
+        if( static_cast<size_t>(lLen) != m_buffer.GetSize() )
+        {
+            PdfRefCountedBuffer temp( lLen );
+            memcpy( temp.GetBuffer(), m_buffer.GetBuffer(), lLen );
+            m_buffer = temp;
+        }
     }
-
-    if( !bLow ) 
-    {
-        // an odd number of bytes was read,
-        // so the last byte is 0
-        *pBuffer++ = cDecodedByte;
-    }
-
-    *pBuffer++ = '\0';
-    *pBuffer++ = '\0';
-
-    // If the allocated internal buffer is too big (e.g. because of whitespaces in the data)
-    // copy to a smaller buffer so that PdfString::GetLength() will be correct
-    lLen = pBuffer - m_buffer.GetBuffer();
-    if( static_cast<size_t>(lLen) != m_buffer.GetSize() )
-    {
-        PdfRefCountedBuffer temp( lLen );
-        memcpy( temp.GetBuffer(), m_buffer.GetBuffer(), lLen );
-        m_buffer = temp;
-    }
-
+    
     if( pEncrypt )
     {
         pdf_long outBufferLen = m_buffer.GetSize() - 2 - pEncrypt->CalculateStreamOffset();
@@ -315,7 +323,11 @@ void PdfString::SetHexData( const char* pszHex, pdf_long lLen, PdfEncrypt* pEncr
                            static_cast<unsigned int>(m_buffer.GetSize()-2),
                           reinterpret_cast<unsigned char*>(outBuffer.GetBuffer()),
                           outBufferLen);
-        outBuffer.Resize(outBufferLen);
+        // Add trailing pair of zeros
+        outBuffer.Resize(outBufferLen + 2);
+        outBuffer.GetBuffer()[outBufferLen] = '\0';
+        outBuffer.GetBuffer()[outBufferLen + 1] = '\0';
+
         // Replace buffer with decrypted value
         m_buffer = outBuffer;
     }
@@ -342,9 +354,9 @@ void PdfString::Write ( PdfOutputDevice* pDevice, EPdfWriteMode eWriteMode, cons
     // Peter Petrov: 17 May 2008
     // Added check - m_buffer.GetSize()
     // Now we are not encrypting the empty strings (was access violation)!
-    if( pEncrypt && m_buffer.GetSize())
+    if( pEncrypt && m_buffer.GetSize() && IsValid() )
     {
-        pdf_long nInputBufferLen = m_buffer.GetSize();
+        pdf_long nInputBufferLen = m_buffer.GetSize() - 2; // Cut off the trailing pair of zeros
         pdf_long nUnicodeMarkerOffet = sizeof( PdfString::s_pszUnicodeMarker );
         if( m_bUnicode )
             nInputBufferLen += nUnicodeMarkerOffet;
@@ -354,10 +366,10 @@ void PdfString::Write ( PdfOutputDevice* pDevice, EPdfWriteMode eWriteMode, cons
         if( m_bUnicode )
         {
             memcpy(pInputBuffer, PdfString::s_pszUnicodeMarker, nUnicodeMarkerOffet);
-            memcpy(&pInputBuffer[nUnicodeMarkerOffet], m_buffer.GetBuffer(), m_buffer.GetSize());
+            memcpy(&pInputBuffer[nUnicodeMarkerOffet], m_buffer.GetBuffer(), nInputBufferLen - nUnicodeMarkerOffet);
         }
         else
-            memcpy(pInputBuffer, m_buffer.GetBuffer(), m_buffer.GetSize());
+            memcpy(pInputBuffer, m_buffer.GetBuffer(), nInputBufferLen);
         
         pdf_long nOutputBufferLen = pEncrypt->CalculateStreamLength(nInputBufferLen);
         
@@ -375,10 +387,10 @@ void PdfString::Write ( PdfOutputDevice* pDevice, EPdfWriteMode eWriteMode, cons
     }
 
     pDevice->Print( m_bHex ? "<" : "(" );
-    if( m_buffer.GetSize() )
+    if( m_buffer.GetSize() && IsValid() )
     {
         char* pBuf = m_buffer.GetBuffer();
-        pdf_long  lLen = m_buffer.GetSize() - 2;
+        pdf_long  lLen = m_buffer.GetSize() - 2; // Cut off the trailing pair of zeros
 
         if( m_bHex ) 
         {
@@ -440,6 +452,12 @@ const PdfString & PdfString::operator=( const PdfString & rhs )
 
 bool PdfString::operator>( const PdfString & rhs ) const
 {
+    if ( !this->IsValid() || !rhs.IsValid() )
+    {
+        PdfError::LogMessage( eLogSeverity_Error, "PdfString::operator> LHS or RHS was invalid PdfString" );
+        return false;
+    }
+    
     const PdfString & str1 = *this;
     const PdfString & str2 = rhs;
 
@@ -463,6 +481,12 @@ bool PdfString::operator>( const PdfString & rhs ) const
 
 bool PdfString::operator<( const PdfString & rhs ) const
 {
+    if ( !this->IsValid() || !rhs.IsValid() )
+    {
+        PdfError::LogMessage( eLogSeverity_Error, "PdfString::operator< LHS or RHS was invalid PdfString" );
+        return false;
+    }
+    
     const PdfString & str1 = *this;
     const PdfString & str2 = rhs;
 
@@ -486,6 +510,17 @@ bool PdfString::operator<( const PdfString & rhs ) const
 
 bool PdfString::operator==( const PdfString & rhs ) const
 {
+    if ( !this->IsValid() && !rhs.IsValid() )
+    {
+        PdfError::LogMessage( eLogSeverity_Error, "PdfString::operator== LHS and RHS both invalid PdfStrings" );
+        return true;
+    }
+    else if ( !this->IsValid() || !rhs.IsValid() )
+    {
+        PdfError::LogMessage( eLogSeverity_Error, "PdfString::operator== LHS or RHS was invalid PdfString" );
+        return false;
+    }
+
     PdfString str1 = *this;
     PdfString str2 = rhs;
 
@@ -525,7 +560,7 @@ void PdfString::Init( const char* pszString, pdf_long lLen )
     }
     
     // skip the first two bytes 
-    if( m_bUnicode )
+    if( m_bUnicode || bUft16LE )
     {
         lLen      -= 2;
         pszString += 2;
@@ -554,7 +589,12 @@ void PdfString::InitFromUtf8( const pdf_utf8* pszStringUtf8, pdf_long lLen )
 
     pdf_long        lBufLen = (lLen << 1) + sizeof(wchar_t);
     // twice as large buffer should always be enough
-    pdf_utf16be *pBuffer = static_cast<pdf_utf16be *>(alloca(lBufLen)); 
+    std::vector<char>	bytes(lBufLen);
+#if (defined(_MSC_VER)  &&  _MSC_VER < 1700) || (defined(__BORLANDC__))	// MSC before VC11 has no data member, same as BorlandC
+    pdf_utf16be *pBuffer = reinterpret_cast<pdf_utf16be *>(&bytes[0]); 
+#else
+    pdf_utf16be *pBuffer = reinterpret_cast<pdf_utf16be *>(bytes.data()); 
+#endif
 
     lBufLen = PdfString::ConvertUTF8toUTF16( pszStringUtf8, lLen, pBuffer, lBufLen );
 
@@ -573,7 +613,7 @@ void PdfString::InitUtf8()
         // UTF8 is at maximum 5 * characterlenght.
 
         pdf_long  lBufferLen = (5*this->GetUnicodeLength())+2;
-        char* pBuffer    = static_cast<char*>(malloc(sizeof(char)*lBufferLen));
+        char* pBuffer    = static_cast<char*>(podofo_calloc( lBufferLen, sizeof(char) ));
         if( !pBuffer )
         {
             PODOFO_RAISE_ERROR( ePdfError_OutOfMemory );
@@ -586,7 +626,7 @@ void PdfString::InitUtf8()
         pBuffer[lUtf8-1] = '\0';
         pBuffer[lUtf8] = '\0';
         m_sUtf8 = pBuffer;
-        free( pBuffer );
+        podofo_free( pBuffer );
     }
     else
     {
@@ -598,6 +638,12 @@ void PdfString::InitUtf8()
 #ifdef _WIN32
 const std::wstring PdfString::GetStringW() const
 {
+    if ( !IsValid() )
+    {
+        PdfError::LogMessage( eLogSeverity_Error, "PdfString::GetStringW invalid PdfString" );
+        return std::wstring();
+    }
+    
     if( !this->IsUnicode() )
     {
         return this->ToUnicode().GetStringW();
@@ -616,7 +662,7 @@ const std::wstring PdfString::GetStringW() const
 
 #endif // _WIN32
 
-/*
+#ifdef PODOFO_PUBLIC_STRING_HEX_CODEC // never set, Decode even says REMOVE :(
 PdfString PdfString::HexEncode() const
 {
     if( this->IsHex() )
@@ -670,20 +716,31 @@ PdfString PdfString::HexDecode() const
 
         return str;
     }
-} 
-*/
+}
+#endif // PODOFO_PUBLIC_STRING_HEX_CODEC 
 
 PdfString PdfString::ToUnicode() const
 {
     if( this->IsUnicode() )
+    {
         return *this;
-    else
+    }
+    else if ( this->IsValid() )
     {
         const PdfEncoding* const pEncoding = (m_pEncoding ? 
                                               m_pEncoding : 
                                               PdfEncodingFactory::GlobalPdfDocEncodingInstance());
         return pEncoding->ConvertToUnicode( *this, NULL );
     }
+    else
+    {
+        // can't convert because PdfString is invalid and has no buffer, so return *this
+        // which means trying to convert an invalid string returns another invalid string
+        // and in the special case where *this is PdfString::StringNull then ToUnicode()
+        // returns PdfString::StringNull
+        PdfError::LogMessage( eLogSeverity_Error, "PdfString::ToUnicode invalid PdfString" );
+        return *this;
+    }
 }
 
 void PdfString::SwapBytes( char* pBuf, pdf_long lLen ) 
