@@ -68,11 +68,11 @@ PdfFontType1::PdfFontType1( PdfFontType1* pFont, PdfFontMetrics* pMetrics, const
 {
 	memset( m_bUsed, 0, sizeof( m_bUsed ) );
 	// don't embedd font
-    Init( false, PdfName("Type1") );
+	Init( false, PdfName("Type1") );
 
 	// Use identical subset-names
 	if ( pFont->IsSubsetting() )
-		GetObject()->GetDictionary().AddKey( "BaseFont", pFont->GetObject()->GetDictionary().GetKey( "BaseFont" ) );
+	  GetObject()->GetDictionary().AddKey( "BaseFont", pFont->GetObject()->GetDictionary().GetKey( "BaseFont" ) );
 
 	// set identifier
 	std::string id = pFont->GetIdentifier().GetName();
@@ -151,7 +151,7 @@ void PdfFontType1::EmbedSubsetFont()
         lSize = ftell( hFile );
         fseek( hFile, 0L, SEEK_SET );
 
-        pAllocated = static_cast<char*>(malloc( sizeof(char) * lSize ));
+        pAllocated = static_cast<char*>(podofo_calloc( lSize, sizeof(char) ));
         if( !pAllocated )
         {
             fclose( hFile );
@@ -387,8 +387,8 @@ void PdfFontType1::EmbedSubsetFont()
 
 	// cleanup memory
     if( pAllocated )
-        free( pAllocated );
-	free( outBuff );
+        podofo_free( pAllocated );
+    delete[] outBuff;
 
 	// enter length in dictionary
     pContents->GetDictionary().AddKey( "Length1", PdfVariant( lLength1 ) );
@@ -441,7 +441,7 @@ void PdfFontType1::EmbedFontFile( PdfObject* pDescriptor )
         lSize = ftell( hFile );
         fseek( hFile, 0L, SEEK_SET );
 
-        pAllocated = static_cast<char*>(malloc( sizeof(char) * lSize ));
+        pAllocated = static_cast<char*>(podofo_calloc( lSize, sizeof(char) ));
         if( !pAllocated )
         {
             fclose( hFile );
@@ -489,13 +489,14 @@ void PdfFontType1::EmbedFontFile( PdfObject* pDescriptor )
 				pBinary = &pBinary[lSegmentLength];
 				break;
 			case 3:									// end-of-file
+			  // First set pContents keys before writing stream, so that PdfTFontType1 works with streamed document
+			        pContents->GetDictionary().AddKey( "Length1", PdfVariant( lLength1 ) );
+				pContents->GetDictionary().AddKey( "Length2", PdfVariant( lLength2 ) );
+				pContents->GetDictionary().AddKey( "Length3", PdfVariant( lLength3 ) );
+
 				pContents->GetStream()->Set( pBuffer, lSize - 2L );
 				if( pAllocated )
-					free( pAllocated );
-
-				pContents->GetDictionary().AddKey( "Length1", PdfVariant( lLength1 ) );
-                pContents->GetDictionary().AddKey( "Length2", PdfVariant( lLength2 ) );
-                pContents->GetDictionary().AddKey( "Length3", PdfVariant( lLength3 ) );
+					podofo_free( pAllocated );
 
 				return;
 			default:
@@ -524,7 +525,7 @@ void PdfFontType1::EmbedFontFile( PdfObject* pDescriptor )
 	// TODO: Pdf Supports only Type1 fonts with binary encrypted sections and not the hex format
 	pContents->GetStream()->Set( pBuffer, lSize );
     if( pAllocated )
-        free( pAllocated );
+        podofo_free( pAllocated );
 
     pContents->GetDictionary().AddKey( "Length1", PdfVariant( lLength1 ) );
     pContents->GetDictionary().AddKey( "Length2", PdfVariant( lLength2 ) );
@@ -671,14 +672,17 @@ pdf_long PdfFontType1::FindInBuffer( const char* pszNeedle, const char* pszHayst
     const char* pszEnd   = pszHaystack + lLen - lNeedleLen; 
     const char* pszStart = pszHaystack;
 
-    while( pszHaystack < pszEnd ) 
+    if ( pszNeedle )
     {
-        if( strncmp( pszHaystack, pszNeedle, lNeedleLen ) == 0 )
-            return pszHaystack - pszStart;
+        while( pszHaystack < pszEnd )
+        {
+            if( strncmp( pszHaystack, pszNeedle, lNeedleLen ) == 0 )
+                return pszHaystack - pszStart;
 
-        ++pszHaystack;
+            ++pszHaystack;
+        }
     }
-
+    
     return -1;
 }
 
@@ -692,9 +696,10 @@ PdfType1EncryptCharstring::PdfType1EncryptCharstring()
     : PdfType1Encrypt()
 {
  	m_r = 4330;
- }
+}
 
 PdfType1Encrypt::PdfType1Encrypt()
+    : m_r( -1 ) // will be initialized in subclasses with real value
 {
 	m_c1 = 52845;
 	m_c2 = 22719;
