@@ -89,7 +89,7 @@ PdfMemDocument::PdfMemDocument( const char* pszFilename )
 }
 
 #ifdef _WIN32
-#if defined(_MSC_VER)  &&  _MSC_VER <= 1200			// nicht fr Visualstudio 6
+#if defined(_MSC_VER)  &&  _MSC_VER <= 1200    // not for MS Visual Studio 6
 #else
 PdfMemDocument::PdfMemDocument( const wchar_t* pszFilename )
     : PdfDocument(), m_pEncrypt( NULL ), m_pParser( NULL )
@@ -194,7 +194,7 @@ void PdfMemDocument::Load( const char* pszFilename )
 }
 
 #ifdef _WIN32
-#if defined(_MSC_VER)  &&  _MSC_VER <= 1200			// nicht fr Visualstudio 6
+#if defined(_MSC_VER)  &&  _MSC_VER <= 1200    // not for MS Visual Studio 6
 #else
 void PdfMemDocument::Load( const wchar_t* pszFilename )
 {
@@ -353,6 +353,9 @@ void PdfMemDocument::SetPassword( const std::string & sPassword )
     m_pParser->SetPassword( sPassword );
     InitFromParser( m_pParser );
     InitPagesTree();
+
+    delete m_pParser;
+    m_pParser = NULL;
 }
 
 void PdfMemDocument::Write( const char* pszFilename )
