@@ -1,4 +1,4 @@
-/***************************************************************************
+/**************************************************************************
  *   Copyright (C) 2006 by Dominik Seichter                                *
  *   domseichter@web.de                                                    *
  *                                                                         *
@@ -16,6 +16,19 @@
  *   License along with this program; if not, write to the                 *
  *   Free Software Foundation, Inc.,                                       *
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
+ *                                                                         *
+ *   In addition, as a special exception, the copyright holders give       *
+ *   permission to link the code of portions of this program with the      *
+ *   OpenSSL library under certain conditions as described in each         *
+ *   individual source file, and distribute linked combinations            *
+ *   including the two.                                                    *
+ *   You must obey the GNU General Public License in all respects          *
+ *   for all of the code used other than OpenSSL.  If you modify           *
+ *   file(s) with this exception, you may extend this exception to your    *
+ *   version of the file(s), but you are not obligated to do so.  If you   *
+ *   do not wish to do so, delete this exception statement from your       *
+ *   version.  If you delete this exception statement from all source      *
+ *   files in the program, then also delete it here.                       *
  ***************************************************************************/
 
 #ifndef _PDF_MEM_DOCUMENT_H_
@@ -23,6 +36,7 @@
 
 #include "podofo/base/PdfDefines.h"
 #include "podofo/base/PdfObject.h"
+#include "podofo/base/PdfExtension.h"
 
 #include "PdfDocument.h"
 #include "PdfFontCache.h"
@@ -70,6 +84,10 @@ class PODOFO_DOC_API PdfMemDocument : public PdfDocument {
      */
     PdfMemDocument();
     
+    /** Construct a new (empty) PdfMemDocument
+     */
+    PdfMemDocument( bool bOnlyTrailer );
+
     /** Construct a PdfMemDocument from an existing PDF (on disk)
      *  \param pszFilename filename of the file which is going to be parsed/opened
      *
@@ -82,7 +100,7 @@ class PODOFO_DOC_API PdfMemDocument : public PdfDocument {
     PdfMemDocument( const char* pszFilename );
 
 #ifdef _WIN32
-#if defined(_MSC_VER)  &&  _MSC_VER <= 1200			// nicht fr Visualstudio 6
+#if defined(_MSC_VER)  &&  _MSC_VER <= 1200    // not for MS Visual Studio 6
 #else
     /** Construct a PdfMemDocument from an existing PDF (on disk)
      *  \param pszFilename filename of the file which is going to be parsed/opened
@@ -118,7 +136,7 @@ class PODOFO_DOC_API PdfMemDocument : public PdfDocument {
     void Load( const char* pszFilename );
 
 #ifdef _WIN32
-#if defined(_MSC_VER)  &&  _MSC_VER <= 1200			// nicht fr Visualstudio 6
+#if defined(_MSC_VER)  &&  _MSC_VER <= 1200    // not for MS Visual Studio 6
 #else
     /** Load a PdfMemDocument from a file
      *
@@ -163,6 +181,10 @@ class PODOFO_DOC_API PdfMemDocument : public PdfDocument {
      */
     void Load( const PdfRefCountedInputDevice & rDevice );
 
+    /** Returns whether the document is fully loaded.
+     */
+    inline bool IsLoaded( void ) const;
+
     /** Writes the complete document to a file
      *
      *  \param pszFilename filename of the document 
@@ -215,6 +237,31 @@ class PODOFO_DOC_API PdfMemDocument : public PdfDocument {
      *  \returns EPdfVersion version of the pdf document
      */
     EPdfVersion GetPdfVersion() const { return m_eVersion; }
+    
+    /** Add a vendor-specific extension to the current PDF version.
+     *  \param ns  namespace of the extension
+     *  \param level  level of the extension
+     */
+    void AddPdfExtension( const char* ns, pdf_int64 level );
+    
+    /** Checks whether the documents is tagged to imlpement a vendor-specific 
+     *  extension to the current PDF version.
+     *  \param ns  namespace of the extension
+     *  \param level  level of the extension
+     */
+    bool HasPdfExtension( const char* ns, pdf_int64 level ) const;
+    
+    /** Remove a vendor-specific extension to the current PDF version.
+     *  \param ns  namespace of the extension
+     *  \param level  level of the extension
+     */
+    void RemovePdfExtension( const char* ns, pdf_int64 level );
+    
+    /** Return the list of all vendor-specific extensions to the current PDF version.
+     *  \param ns  namespace of the extension
+     *  \param level  level of the extension
+     */
+    std::vector<PdfExtension> GetPdfExtensions() const;
 
     /** If you try to open an encrypted PDF file, which requires
      *  a password to open, PoDoFo will throw a PdfError( ePdfError_InvalidPassword ) 
@@ -241,8 +288,8 @@ class PODOFO_DOC_API PdfMemDocument : public PdfDocument {
      *  \param protection several EPdfPermissions values or'ed together to set 
      *                    the users permissions for this document
      *  \param eAlgorithm the revision of the encryption algorithm to be used
-     *  \param eKeyLength the length of the encryption key ranging from 40 to 128 bits 
-     *                    (only used if eAlgorithm == ePdfEncryptAlgorithm_RC4V2)
+     *  \param eKeyLength the length of the encryption key ranging from 40 to 256 bits 
+     *                    (only used if eAlgorithm >= ePdfEncryptAlgorithm_RC4V2)
      *
      *  \see PdfEncrypt
      */
@@ -256,7 +303,7 @@ class PODOFO_DOC_API PdfMemDocument : public PdfDocument {
                                         PdfEncrypt::ePdfPermissions_Accessible |
                                         PdfEncrypt::ePdfPermissions_DocAssembly |
                                         PdfEncrypt::ePdfPermissions_HighPrint,
-                       PdfEncrypt::EPdfEncryptAlgorithm eAlgorithm = PdfEncrypt::ePdfEncryptAlgorithm_RC4V1, 
+                       PdfEncrypt::EPdfEncryptAlgorithm eAlgorithm = PdfEncrypt::ePdfEncryptAlgorithm_RC4V1,
                        PdfEncrypt::EPdfKeyLength eKeyLength = PdfEncrypt::ePdfKeyLength_40 );
 
     /** Encrypt the document during writing using a PdfEncrypt object
@@ -323,6 +370,11 @@ class PODOFO_DOC_API PdfMemDocument : public PdfDocument {
      */
     PdfObject* GetMarkInfo() const { return GetNamedObjectFromCatalog( "MarkInfo" ); }
 
+    /** Get access to the RFC 3066 natural language id for the document (ISO 32000-1:2008 14.9.2.1)
+     *  \returns PdfObject the language ID string
+     */
+    PdfObject* GetLanguage() const { return GetNamedObjectFromCatalog( "Lang" ); }
+
     /** Creates a PdfFont object from an existing font.
      *
      *  \param pObject a PdfObject that is a font
@@ -467,18 +519,27 @@ class PODOFO_DOC_API PdfMemDocument : public PdfDocument {
      */
     inline const PdfEncrypt* GetEncrypt() const;
 
- private:
+//begin L.K
+protected:
+    PdfObject* GetMainTrailer() { return PdfDocument::GetTrailer(); }
+
+    //private
     /** Get a dictioary from the catalog dictionary by its name.
      *  \param pszName will be converted into a PdfName
      *  \returns the dictionary if it was found or NULL
      */
     PdfObject* GetNamedObjectFromCatalog( const char* pszName ) const;
 
+    //private
     /** Internal method to load all objects from a PdfParser object.
      *  The objects will be removed from the parser and are now
      *  owned by the PdfMemDocument.
      */
-    void InitFromParser( PdfParser* pParser );
+    //void InitFromParser( PdfParser* pParser );
+    virtual void InitFromParser( PdfParser* pParser );
+//end L.K
+
+ private:
 
     /** Clear all internal variables
      */
@@ -508,6 +569,14 @@ class PODOFO_DOC_API PdfMemDocument : public PdfDocument {
     EPdfWriteMode   m_eWriteMode;
 };
 
+// -----------------------------------------------------
+// 
+// -----------------------------------------------------
+bool PdfMemDocument::IsLoaded( void ) const
+{
+    return m_pParser == NULL;
+}
+
 // -----------------------------------------------------
 // 
 // -----------------------------------------------------
