@@ -67,6 +67,7 @@ void PdfSignOutputDevice::Init()
     m_pSignatureBeacon = NULL;
     m_bBeaconFound = false;
     m_bDevOwner = false;
+    m_sBeaconPos = 0;
 }
 
 PdfSignOutputDevice::~PdfSignOutputDevice()
@@ -89,13 +90,18 @@ void PdfSignOutputDevice::SetSignatureSize(size_t lSignatureSize)
     size_t lLen = sizeof(srcBeacon);
 
 	lSignatureSize = 2*lSignatureSize;
-    char* pData = static_cast<char*>(malloc(lSignatureSize));
+    char* pData = static_cast<char*>(podofo_malloc(lSignatureSize));
+ 	if (!pData)
+ 	{
+ 		PODOFO_RAISE_ERROR(ePdfError_OutOfMemory);
+    }
+    
     for(size_t i=0; i<lSignatureSize; i++)
     {
         pData[i]=srcBeacon[i%lLen];
     }
     m_pSignatureBeacon = new PdfData(pData, lSignatureSize);
-    free(pData);
+    podofo_free(pData);
 }
 
 size_t PdfSignOutputDevice::GetSignatureSize()const
@@ -161,12 +167,23 @@ void PdfSignOutputDevice::AdjustByteRange()
     m_pRealDevice->Seek(m_sBeaconPos-sPosition.size()-9);
     char ch;
     size_t offset = m_pRealDevice->Tell();
-    /* Sanity test, the file position should be at the '[' now */
-    if (m_pRealDevice->Read(&ch, 1) != 1 || ch != '[') {
-        PODOFO_RAISE_ERROR( ePdfError_InternalLogic );
-    } else {
-       m_pRealDevice->Seek(offset);
+	size_t size;
+
+    /* Sanity tests... */
+    size = m_pRealDevice->Read(&ch, 1);
+    PODOFO_RAISE_LOGIC_IF( size != 1, "Failed to read 1 byte." );
+    if (ch == '0') {
+       /* probably clean write mode, whic means two more bytes back */
+       m_pRealDevice->Seek(m_sBeaconPos-sPosition.size()-11);
+       offset = m_pRealDevice->Tell();
+       size = m_pRealDevice->Read(&ch, 1);
+       PODOFO_RAISE_LOGIC_IF( size != 1, "Failed to read 1 byte." );
     }
+
+    /* ...the file position should be at the '[' now */
+    PODOFO_RAISE_LOGIC_IF( ch != '[', "Failed to find byte range array start in the stream." );
+
+    m_pRealDevice->Seek(offset);
     m_pRealDevice->Write(sPosition.c_str(), sPosition.size());
 }
 
